/***************************************************************************

main.cpp

Windows -> Dynamic library (dll) 

Linux or MacOS -> Dynamic shared object (so, dylib)

-------------------

begin : Wed Dec 1 2010

copyright : (C) 2010 by URIOT Didier

email : duriot@cea.fr

***************************************************************************/

#ifdef _WIN32

  #include <windows.h>

  #define DLL_EXPORT __declspec(dllexport)



#else

  #define DLL_EXPORT

#endif

#include <cmath>

#include <cstdio>

#include <cstdlib>

#include <cstring>

#ifdef __cplusplus

extern "C" {

#endif



 
//--------------------------------------------------------------------------------

//-- MY_DIAG - MY_DIAG - MY_DIAG - MY_DIAG - MY_DIAG - MY_DIAG - MY_DIAG ---------

//-- MY_DIAG - MY_DIAG - MY_DIAG - MY_DIAG - MY_DIAG - MY_DIAG - MY_DIAG ---------

//--------------------------------------------------------------------------------

//

// Syntax in struture.dat file

//

// MY_DIAG(my_diag_name)(Weight) Diag_number target arg2,arg3...arg10

// Diag_number is the number to associated to ADJUST commands 

// “(Weight)” is optional

//

// following functions have to be defined

//  - my_diag_name_partran   : for the beam tracking treatment

//  - my_diag_name_envelope  : for the beam envelope treatment

//

// If "my_diag_name" is not defined (MY_DIAG target arg2,arg3...arg10)

// 2 functions are defined

//  - my_diag_partran   : for the beam tracking treatment

//  - my_diag_envelope  : for the beam envelope treatment

//

// TraceWin Criteria = pow(target_value - diag_value),2)

//

//

// below you can find an example of a beam position or beam size measurment :

// Syntax: MY_DIAG(my_pos_and_size) target arg2,arg3...arg10

// following functions have to be defined

//  - my_pos_and_size_partran   : for the beam tracking treatment

//  - my_pos_and_size_envelope  : for the beam envelope treatment

 

// You have to compile this example as an dll

// - my_elements.dll   (for windows)

// - my_elements.so    (for linux)

// - my_elements.dylib (for MacOS)

 

// Commands to compil and link GNU gcc compiler:

// g++.exe m32 -Wall -c main.cpp -o main.o

// g++.exe m32 -shared -Wl,--dll main.o -o my_elements.dll

// If you use 64bits TraceWin version replace both -m32 by -m64

 


// MY_DIAG Multiparticle

int DLL_EXPORT my_pos_and_size_partran(double *diag_value,double *param,int Nele,int npart,double *cord,double *loss,double freq,double mass0,int q,double ws,double Ibeam,double *extra_param,char *error_mess)

{

  double x2,y2,x,y,xp,yp,z,dpsp,xmoy,ymox;

  int ng;

  //

  // param[1]    : target_value

  // param[2]    : you can use this second parameter to define to diagnostic types

  // param[3->8] : free

  // diag_value  : value of your diagnostic

  // cord        : See example

  // loss        : see example

  // freq        : beam frequency (Hz)

  // mass        : Particle mass (eV)

  // q           : particle charge state

  // ws          : reference kinetic energy (eV) (can be modified)

  // Ibeam       : Beam current (A) (can be modified)

  // error_mess  : TraceWin stop and show this error message if this function return 0

  // if error_mess!="" and function return 1, this message is print to the standart console without stop TraceWin

 

  strcpy(error_mess,"");

  xmoy=x2=0;
  ymox=y2=0;

  ng=0;

  for (int i=0;i<npart;i++) {

    if ((int)loss[i]==0) {

      x=cord[i*6];      // m

      xp=cord[i*6+1];   // rad

      y=cord[i*6+2];    // m

      yp=cord[i*6+3];   // rad

      z=cord[i*6+4];    // m

      dpsp=cord[i*6+5]; // dp/p

 

      x2+=x*x;

      xmoy+=x;

      y2+=y*y;

      ymox+=y;


      ng++;

    }

  }

  if (param[3]==0) {

    if ( ng >0) {  
        xmoy/=ng;
        ymox/=ng;
    }  
    else{ 
        xmoy=1e6;
        ymox=1e6;

    diag_value[0]=xmoy;  // X position (m)
    diag_value[1]=ymox;  // Y position (m)
    }
  }

  if (param[3]==1) {

    x2=sqrt(x2/ng);
    y2=sqrt(y2/ng);

    diag_value[0]=x2;  // X size (m)
    diag_value[1]=y2;  // Y size (m)
  }


  //sprintf(error_mess,"%lg %d %lg %lg",param[2],ng,x2,*diag_value);

 

  return(1);

}

 

//--------------------------------------------------------------------------------------------------------------------------------------------------------------------

 

// MY_DIAG Envelope

// MY_DIAG Envelope

int DLL_EXPORT my_pos_and_size_envelope(double *diag_value,double *param,int Nele,double Bmat[][6],double *Bcent,double freq,double mass0,int q,double ws,double ibeam,double *extra_param,char *error_mess)

{

  //

  // param[1]    : target_value

  // param[2]    : you can use this second parameter to define to diagnostic types

  // param[3->8] : free

  // diag_value  : value of your diagnostic

  // Nele        : Element number

  // Bmat        : Beam matrix 6x6 (x(m), xp(rad, y(m), yp(rad), z(m), dp/p)

  // Bcent       : beam centroid vector (x(m), xp(rad, y(m), yp(rad), z(m), dp/p)

  // freq        : beam frequency (Hz)

  // mass        : Particle mass (eV)

  // q           : particle charge state

  // ws          : reference kinetic energy (eV) (can be modified)

  // Ibeam       : Beam current (A) (can be modified)

  // error_mess  : TraceWin stop and show this error message if this function return 0

  // if error_mess!="" and function return 1, this message is print to the standart console without stop TraceWin

 

  printf("POS ENV\n");

 

  if (param[3]==0) {

    diag_value[0]=Bcent[0];   // X beam position (m)
    diag_value[1]=Bcent[2];   // X beam position (m)
  }

  if (param[3]==1) {

    diag_value[0]=sqrt(Bmat[0][0]);  // beam size (X); (m)
    diag_value[1]=sqrt(Bmat[2][2]);  // beam size (Y); (m)
  }

  return(1);

}

 

#ifdef __cplusplus

} // "C"

#endif

 

 

 

